% Main function for computing closure
% Most of the function contains experimental stuff.
% Tested setting for parameters are:
% recursive_on = false; do_padding = true; alg_type = 'area';
function [X, Xs, cost, sup_image, costs] = grabSuperpixeInfo(img_filename, sup_image, image_data, alg_type,  selected_sups, edge_thresh)
       
 
    
    try
        img = im2double(imread(img_filename));
    catch
        img = img_filename;
    end


    if (~exist('edge_thresh', 'var'))
        edge_thresh = [];
    end
   
    num_sups = max(sup_image(:));
    adjmat = SuperpixelAdjacencyGraphNcuts(sup_image);
        
    if (exist('selected_sups', 'var') && ~isempty(selected_sups))
        num_sups = sum(selected_sups);
        adjmat = adjmat(selected_sups, selected_sups);
        sup_image(SupValueImage_MEX(sup_image, double(selected_sups)) == 0) = 0;
        [b,m,new_sups] = unique(sup_image(:));
        sup_image = reshape(new_sups, size(sup_image)) - 1;
    end    

    % Compute all the edge fragments between superpixels
    
    % Include (or not) the fragments around the image border
    do_padding = true;
    if (do_padding)
        s = padarray(sup_image, [1,1]);
        s(s == 0) = num_sups+1;
        [fragments, junctions, neighbor_data, sup_image, avgcolor, polyfragments, poly_params, T] = seg2fragments(s, padarray(img,[2,2,0]), 10, 3);
        
        [poly_angle, poly_tangent, poly_sq_curv] = polyfragments_angle_and_curvature(T, poly_params);

        fragments = cellfun(@(x) (x - 1), fragments, 'UniformOutput', false);
        sup_image = sup_image(2:end-1, 2:end-1);
        sup_image(sup_image == (num_sups+1)) = 0;
        segment_fragments = neighbor_data.segment_fragments(1:end-1);
        num_sups = numel(segment_fragments);

        fragment_segments = cellfun(@(x) (x(x <= num_sups)), ...
            neighbor_data.fragment_segments, 'UniformOutput', false);
    else
        [fragments, junctions, neighbor_data, sup_image, avgcolor, polyfragments, poly_params, T] = seg2fragments(sup_image, img, 10, 3);
        [poly_angle, poly_tangent, poly_sq_curv] = polyfragments_angle_and_curvature(T, poly_params);

        segment_fragments = neighbor_data.segment_fragments;
        num_sups = numel(segment_fragments);

        fragment_segments = neighbor_data.fragment_segments;
    end
    
    STATS = regionprops(sup_image, 'Area');
    A = diag([STATS(:).Area]);
    
    
    gap_multiplier = ones(size(A));
    edge_affinity = ones(size(A));
    
    % Choose the closure cost type (currently only 'area' is well tested)
    try
        if (exist('alg_type', 'var') && ~isempty(alg_type))
            switch alg_type
                case 'area'
                    affinity = A;
                case 'appearance_affinity'
                    fg = double(rand(size(sup_image)) > 0.5);
                    [fg_prob, bg_prob, rgbhist] = ForegroundBackgroundProbColorHist(img, fg, linspace(0,1,5), linspace(0,1,5), linspace(0,1,5));
                    hist_affinities = SuperpixelAffinityHistogram(sup_image, adjmat, rgbhist, 125);
                    affinity = hist_affinities;
    %                 gap_multiplier(eye(size(adjmat)) == 0) = affinity(eye(size(adjmat)) == 0);
                case 'junction_affinity'
                    [affinity_junc, affinity_junc_on] = Junc3AffinityOne(img, sup_image, image_data);
                    affinity = (affinity_junc .* adjmat);
    %                 contour_affinity = ContourAffinity(affinity, neighbor_data, fragment_segments);
    %                 edge_affinity = contour_affinity;
    %                 [D, P, L, maxEdge] = ShortestPaths(affinity);
    %                 maxD = max(D(~isinf(D)));
    %                 D(isinf(D)) = maxD;
    %                 D = D / maxD;
    %                 affinity = exp(-D*10);
    %                 affinity(eye(size(affinity))==1) = 0;

                case 'junction_and_area'
                    affinity = affinity_junc .* (diag(A)*diag(A)');
                case 'junction_area_weighted_gap'
                    affinity = (affinity_junc .* adjmat);
                    gap_multiplier = (diag(A)*diag(A)');
                otherwise
                    error('Unknown affinity');
            end
        else
            affinity = A;
        end
    catch
        error('Unsupported affinity type');
    end
    
         
    %% superpixel closure  
    [E, P, frag_edge, frag_per] = SuperpixelAffinityEdgeContrastGivenFragments(fragments, segment_fragments, fragment_segments, ...
        poly_angle, poly_sq_curv, image_data, edge_thresh, edge_affinity);    
   % G = P - E;
   % G = G .* gap_multiplier;
   % D = sum(affinity,2);
%     X = MinimizeRatioBinary(double(rand(num_sups,1)>0.5),G, affinity);
%     
    scFile = [img_filename(1:end-4),'_sc','.mat'];
 save (scFile, 'fragments', 'junctions', 'neighbor_data', 'segment_fragments',...
     'fragment_segments', 'frag_edge',  'frag_per', 'sup_image');
end

